import React from 'react';
import { match, RouterContext } from 'react-router';
import ReactDOM from 'react-dom';
import ReactDOMServer from 'react-dom/server';
import VERSION from './version';

class ReactRouterIntegration {
  constructor() {
    this.version = VERSION;
    this.routers = {};
    this.enabled = false;
    this.register = this.register.bind(this);
    this.get = this.get.bind(this);
    this.render = this.render.bind(this);
  }

  register(name, route) {
    this.routers[name] = route;
  }

  get(name) {
    return this.routers[name];
  }

  render(name, node, props = {}) {
    if (this.enabled === true) {
      throw new Error(
        `Error when rendering ${name}\n
        \trenderRouter: can't render more than one router on the same page.`
      );
    }
    this.enabled = true;
    ReactDOM.render(this._initializeRouter(name, props), node);
  }

  unmount(node) {
    ReactDOM.unmountComponentAtNode(node);
    this.enabled = false;
  }

  _initializeRouter(name, props) {
    const constructor = this.get(name);
    return typeof(constructor) === 'function' ? constructor(props) : constructor;
  }

  renderRouterToString(routerName, props, href) {
    const routes = this._initializeRouter(routerName, props);
    const result = {
      body: '',
      code: 0,
    };

    match({ routes, location: href }, (error, redirectLocation, renderProps) => {
      if (error) {
        throw error;
      } else if (redirectLocation) {
        result.code = 302;
        result.redirectUri = `${redirectLocation.pathname}${redirectLocation.search}`;
      } else if (renderProps) {
        result.body = ReactDOMServer.renderToString(<RouterContext {...renderProps} />);
        result.code = 200;
      } else {
        result.code = 404;
      }
    });
    return JSON.stringify(result);
  }

  get integrationWrapper() {
    return {
      mount: function _mount(node, payload) {
        this.render(payload.name, node, payload.props);
      }.bind(this),

      unmount: function _unmount(node) {
        this.unmount(node);
      }.bind(this),

      nodeRun: function _prerender(payload) {
        return this.renderRouterToString(payload.name, payload.props, payload.path);
      }.bind(this),
    };
  }
}

export default new ReactRouterIntegration;
