module ReactWebpackRails
  module ReactRouterIntegration
    module ViewHelpers
      def react_router(name, props = {}, options = {})
        payload = { name: name, props: props }
        result = ''
        if server_side?(options.delete(:server_side))
          result = execute_router_server_side!(payload, request.path)
        end
        react_element('react-router', payload, options) { result }
      end

      private

      def routing_error(name, path)
        "RactRouter: No route found in #{name} router for #{path}."
      end

      def server_side?(server_side)
        server_side.nil? ? Rails.application.config.react.server_side : server_side
      end

      def execute_router_server_side!(payload, path)
        runner = NodeIntegrationRunner.new('react-router', payload.merge(path: path))
        result = JSON.parse(runner.run)

        case result['code']
        when 200
          result['body'].html_safe
        when 302
          controller.redirect_to result['redirectUri']
        else
          fail ActionController::RoutingError, routing_error(payload[:name], path)
        end
      end
    end
  end
end
